@@ -1,6 +1,6 @@
 /* write.c - emit .o file
    Copyright 1986, 1987, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997,
-   1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
+   1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
@@ -1084,6 +1084,15 @@ install_reloc (asection *sec, arelent *reloc, fragS *fragp,
 {
   char *err;
   bfd_reloc_status_type s;
+  asymbol *sym;
+
+  if (reloc->sym_ptr_ptr != NULL
+      && (sym = *reloc->sym_ptr_ptr) != NULL
+      && (sym->flags & BSF_KEEP) == 0
+      && ((sym->flags & BSF_SECTION_SYM) == 0
+	  || (EMIT_SECTION_SYMBOLS
+	      && !bfd_is_abs_section (sym->section))))
+    as_bad_where (file, line, _("redefined symbol cannot be used on reloc"));
 
   s = bfd_install_relocation (stdoutput, reloc,
 			      fragp->fr_literal, fragp->fr_address,
@@ -1377,6 +1386,10 @@ set_symtab (void)
       for (i = 0; i < nsyms; i++, symp = symbol_next (symp))
 	{
 	  asympp[i] = symbol_get_bfdsym (symp);
+	  if (asympp[i]->flags != BSF_SECTION_SYM
+	      || !(bfd_is_const_section (asympp[i]->section)
+		   && asympp[i]->section->symbol == asympp[i]))
+	    asympp[i]->flags |= BSF_KEEP;
 	  symbol_mark_written (symp);
 	}
     }
@@ -2305,7 +2318,7 @@ relax_segment (struct frag *segment_frag_root, segT segment, int pass)
 		    }
 
 		  know (fragP->fr_next);
-		  after = fragP->fr_next->fr_address;
+		  after = fragP->fr_next->fr_address + stretch;
 		  growth = target - after;
 		  if (growth < 0)
 		    {
@@ -2340,14 +2353,10 @@ relax_segment (struct frag *segment_frag_root, segT segment, int pass)
 		      fragP->fr_type = rs_align;
 		      fragP->fr_subtype = 0;
 		      fragP->fr_offset = 0;
-		      fragP->fr_fix = after - was_address;
-		      break;
+		      fragP->fr_fix = after - address;
 		    }
-
-		  /* This is an absolute growth factor  */
-		  growth -= stretch;
-		  break;
 		}
+		break;
 
 	      case rs_space:
 		growth = 0;
@@ -2520,7 +2529,8 @@ print_fixup (fixS *fixp)
   fprintf_vma (stderr, (bfd_vma) ((bfd_hostptr_t) fixp->fx_frag));
   fprintf (stderr, " where=%ld offset=%lx addnumber=%lx",
 	   (long) fixp->fx_where,
-	   (long) fixp->fx_offset, (long) fixp->fx_addnumber);
+	   (unsigned long) fixp->fx_offset,
+	   (unsigned long) fixp->fx_addnumber);
   fprintf (stderr, "\n    %s (%d)", bfd_get_reloc_code_name (fixp->fx_r_type),
 	   fixp->fx_r_type);
   if (fixp->fx_addsy)
